Package: tinytable
Type: Package
Title: Simple and Configurable Tables in 'HTML', 'LaTeX', 'Markdown',
        'Word', 'PNG', 'PDF', and 'Typst' Formats
Description: Create highly customized tables with this simple and dependency-free package. Data frames can be converted to 'HTML', 'LaTeX', 'Markdown', 'Word', 'PNG', 'PDF', or 'Typst' tables. The user interface is minimalist and easy to learn. The syntax is concise. 'HTML' tables can be customized using the flexible 'Bootstrap' framework, and 'LaTeX' code with the 'tabularray' package.
Version: 0.18.0
Imports: methods
Depends: R (>= 4.1.0)
Enhances: knitr
Suggests: base64enc, broom, data.table (>= 1.15.2), estimatr, ggplot2,
        gh, glue, htmltools, litedown (>= 0.6), magrittr,
        marginaleffects, modelsummary, pandoc, pkgdown, png, quarto,
        Rdatasets, rmarkdown, rstudioapi, scales, stringi, tibble,
        tinysnapshot (>= 0.2.0), tinytest, tinytex, xfun (>= 0.51),
        webshot2
URL: https://vincentarelbundock.github.io/tinytable/
BugReports: https://github.com/vincentarelbundock/tinytable/issues
Authors@R: c(
  person("Vincent", "Arel-Bundock",
  email = "vincent.arel-bundock@umontreal.ca",
  role = c("aut", "cre"),
  comment = c(ORCID = "0000-0003-2042-7063")))
License: GPL (>= 3)
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-19 20:33:12 UTC; vincent
Author: Vincent Arel-Bundock [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2042-7063>)
Maintainer: Vincent Arel-Bundock <vincent.arel-bundock@umontreal.ca>
Repository: CRAN
Date/Publication: 2026-08-20 05:20:21 UTC
