# Generated by roxygen2: do not edit by hand

S3method(knitr::knit_print, tinytable)
S3method(print,tinytable)
S3method(subset,tinytable)
S3method(tt,data.frame)
S3method(tt,data.table)
S3method(tt,default)
S3method(tt,tbl_df)
S3method(xfun::record_print,tinytable)
export(format_tt)
export(format_vector)
export(group_tt)
export(knit_print.tinytable)
export(pkgdown_print.tinytable)
export(plot_tt)
export(plot_vector)
export(save_tt)
export(style_tt)
export(style_vector)
export(theme_default)
export(theme_empty)
export(theme_grid)
export(theme_html)
export(theme_latex)
export(theme_markdown)
export(theme_revealjs)
export(theme_rotate)
export(theme_striped)
export(theme_tinytable)
export(theme_tt)
export(theme_typst)
export(tt)
export(tt_format)
export(tt_group)
export(tt_plot)
export(tt_save)
export(tt_style)
exportClasses(tinytable)
exportMethods("colnames<-")
exportMethods("names<-")
exportMethods(colnames)
exportMethods(names)
exportMethods(rbind2)
importFrom(methods,rbind2)
