# replaced this with @rawNamespace S3method(knitr::knit_print, tinytable) in print.R

# .onLoad <- function(libname, pkgname) {
#   if (requireNamespace("knitr", quietly = TRUE)) {
#     registerS3method("knit_print", "tinytable", knit_print.tinytable, envir = asNamespace("knitr"))
#   }
# }
