% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa_class.R
\docType{class}
\name{tinytable-class}
\alias{tinytable-class}
\title{tinytable S4 class}
\description{
tinytable S4 class
}
\keyword{internal}
