% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_tt.R
\name{theme_tt}
\alias{theme_tt}
\title{Deprecated: Use format-specific theme functions instead}
\usage{
theme_tt(x, theme, ...)
}
\arguments{
\item{x}{deprecated}

\item{theme}{deprecated}

\item{...}{Additional arguments}
}
\value{
Throws an informative error message
}
\description{
\strong{DEPRECATED}: The \code{theme_tt()} function has been deprecated. Please use the format-specific or style-specific theme functions instead.
}
