% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa_class.R
\name{group_eval_j}
\alias{group_eval_j}
\title{Apply group settings to a tinytable}
\usage{
group_eval_j(x, ...)
}
\arguments{
\item{x}{A data frame, data table, or tibble to be rendered as a table.}

\item{...}{Additional arguments are ignored}
}
\description{
Apply group settings to a tinytable
}
\keyword{internal}
